<?php

namespace LatePoint\GoogleCalendarAddon;

use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32;
use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex;
use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base64;
use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base64DotSlash;
use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base64DotSlashOrdered;
use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base64UrlSafe;
use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding;
use LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Hex;
class EncodingTest extends \LatePoint\GoogleCalendarAddon\PHPUnit\Framework\TestCase
{
    public function testBase32Encode()
    {
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("\0"), 'aa======');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("\0\0"), 'aaaa====');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("\0\0\0"), 'aaaaa===');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("\0\0\0\0"), 'aaaaaaa=');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("\0\0\0\0\0"), 'aaaaaaaa');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("\0\0\17"), 'aaaa7777');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("\0\0"), '7777aaaa');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("s9"), 'zzzzzzzz');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode("ֵkZ"), '22222222');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("\0"), 'AA======');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("\0\0"), 'AAAA====');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("\0\0\0"), 'AAAAA===');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("\0\0\0\0"), 'AAAAAAA=');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("\0\0\0\0\0"), 'AAAAAAAA');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("\0\0\17"), 'AAAA7777');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("\0\0"), '7777AAAA');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("s9"), 'ZZZZZZZZ');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32::encodeUpper("ֵkZ"), '22222222');
    }
    public function testBase32Hex()
    {
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex::encode("\0"), '00======');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex::encode("\0\0"), '0000====');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex::encode("\0\0\0"), '00000===');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex::encode("\0\0\0\0"), '0000000=');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex::encode("\0\0\0\0\0"), '00000000');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex::encode("\0\0\17"), '0000vvvv');
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base32Hex::encode("\0\0"), 'vvvv0000');
    }
    /**
     * Based on test vectors from RFC 4648
     */
    public function testBase32Decode()
    {
        $this->assertSame("\0\0\0\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaaaaaaa======'));
        $this->assertSame("\0\0\0\0\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaaaaaaaaa===='));
        $this->assertSame("\0\0\0\0\0\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaaaaaaaaaa==='));
        $this->assertSame("\0\0\0\0\0\0\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaaaaaaaaaaaa='));
        $this->assertSame("\0\0\0\0\0\0\0\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaaaaaaaaaaaaa'));
        $this->assertSame("\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aa======'));
        $this->assertSame("\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaa===='));
        $this->assertSame("\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaa==='));
        $this->assertSame("\0\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaaaa='));
        $this->assertSame("\0\0\0\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaaaaaa'));
        $this->assertSame("\0\0\17", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('aaaa7777'));
        $this->assertSame("\0\0", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('7777aaaa'));
        $this->assertSame("s9", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('zzzzzzzz'));
        $this->assertSame("ֵkZ", \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('22222222'));
        $this->assertSame('foobar', \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode('mzxw6ytboi======'));
        $rand = \random_bytes(9);
        $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode($rand);
        $this->assertSame(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode($rand), \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode($enc)));
        $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode($enc));
    }
    /**
     * @covers Encoding::hexDecode()
     * @covers Encoding::hexEncode()
     * @covers Encoding::base32Decode()
     * @covers Encoding::base32Encode()
     * @covers Encoding::base64Decode()
     * @covers Encoding::base64Encode()
     * @covers Encoding::base64DotSlashDecode()
     * @covers Encoding::base64DotSlashEncode()
     * @covers Encoding::base64DotSlashOrderedDecode()
     * @covers Encoding::base64DotSlashOrderedEncode()
     */
    public function testBasicEncoding()
    {
        // Re-run the test at least 3 times for each length
        for ($j = 0; $j < 3; ++$j) {
            for ($i = 1; $i < 84; ++$i) {
                $rand = \random_bytes($i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::hexEncode($rand);
                $this->assertSame(\bin2hex($rand), $enc, "Hex Encoding - Length: " . $i);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::hexDecode($enc), "Hex Encoding - Length: " . $i);
                // Uppercase variant:
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Hex::encodeUpper($rand);
                $this->assertSame(\strtoupper(\bin2hex($rand)), $enc, "Hex Encoding - Length: " . $i);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Hex::decode($enc), "HexUpper Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Encode($rand);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32Decode($enc), "Base32 Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32EncodeUpper($rand);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32DecodeUpper($enc), "Base32Upper Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32HexEncode($rand);
                $this->assertSame(\bin2hex($rand), \bin2hex(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32HexDecode($enc)), "Base32Hex Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32HexEncodeUpper($rand);
                $this->assertSame(\bin2hex($rand), \bin2hex(\LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base32HexDecodeUpper($enc)), "Base32HexUpper Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base64Encode($rand);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base64Decode($enc), "Base64 Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base64EncodeDotSlash($rand);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base64DecodeDotSlash($enc), "Base64 DotSlash Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base64EncodeDotSlashOrdered($rand);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Encoding::base64DecodeDotSlashOrdered($enc), "Base64 Ordered DotSlash Encoding - Length: " . $i);
                $enc = \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base64UrlSafe::encode($rand);
                $this->assertSame(\strtr(\base64_encode($rand), '+/', '-_'), $enc);
                $this->assertSame($rand, \LatePoint\GoogleCalendarAddon\ParagonIE\ConstantTime\Base64UrlSafe::decode($enc));
            }
        }
    }
}
